#!/usr/bin/python

from mod_utils   import Choice
from mod_elm     import ELM
import sys
import os


class CarEcus:
  '''List all possible ECUs of car'''
  
  allecus = [];       #all ecus possible in this model
  detectedEcus = [];  #ecus detected in car during scan
  vhcls  = [];
  ecus   = [];
  models = [];
  reqres = [];        #results of previous requests
  currentEcu = 0;
  def __init__(self, elm_ref):
    self.elm = elm_ref
    self.s = ";"
    
    mf = open("dat_cars.csv", "rt")
    ef = open("dat_ecus.csv", "rt")
    
    self.vhcls = mf.readlines()
    self.ecus  = ef.readlines()
  
  def scanAllEcus(self):
    '''scan all ecus'''
     
    self.reqres = []
    self.errres = []
    
    i = 0
    print '\r'+'\t\t\t\t'+'\rScanning:'+str(i)+'/'+str(len(self.allecus))+" Detected: "+str(len(self.detectedEcus)),
    sys.stdout.flush()
    
    #scan CAN ecus
    self.elm.init_can()
    for row in self.allecus:
      rowl = row.split(";")
      if rowl[1]=='can' and rowl[2]=='6' and rowl[3]=='14':
        
        i = i+1
        print '\r'+'\t\t\t\t'+'\rScanning:'+str(i)+'/'+str(len(self.allecus))+" Detected: "+str(len(self.detectedEcus)),
        sys.stdout.flush()
        
        self.elm.set_can_addr( rowl[9] )
        self.scan_can( rowl )

    self.elm.close_protocol()

    #scan KWP ecud
    self.elm.init_iso()
    for row in self.allecus:
      rowl = row.split(";")
      if rowl[1]=='iso' and rowl[2]=='7' and rowl[3]=='15':
        
        i = i+1
        print '\r'+'\t\t\t\t'+'\rScanning:'+str(i)+'/'+str(len(self.allecus))+" Detected: "+str(len(self.detectedEcus)),
        sys.stdout.flush()
   
        self.elm.set_iso_addr( rowl[9] )
        self.scan_iso( rowl )
    print '\r'+'\t\t\t\t'+'\rScanning:'+str(i)+'/'+str(len(self.allecus))+" Detected: "+str(len(self.detectedEcus)),
        
  def chooseECU(self, necu):
    
    if len(self.detectedEcus)==0:
      self.scanAllEcus()
    
    if len(self.detectedEcus)==0:
      print "NO ECU detected. Nothing to do. ((("
      exit( 2 )
    
    listecu = []
    
    print "\n    %-4s %-6s %-5s %-25s %s" % ("Addr","Family","Index","Name","Errors")
    for row in self.detectedEcus:
      line = "%-4s %-6s %-5s %-25s %s" % (row[4], row[0],row[1],row[2],row[3])
      listecu.append( line )
    listecu.append( "Exit" )    
    choice = Choice(listecu, "Please choose ECU :")
    if choice[0]=="Exit": exit(1)
    i = int(choice[1])-1
    self.currentEcu = i
    if len(self.detectedEcus[i][0])==1:
      self.detectedEcus[i][0] = '0'+self.detectedEcus[i][0]
    ecuindex = self.detectedEcus[i][0]+self.detectedEcus[i][1]    
    return self.detectedEcus[self.currentEcu]
  
  def getCurrentEcu(self):
    return self.detectedEcus[self.currentEcu]
    
  def chooseModel(self, num):
    
    for str1 in self.vhcls:
      model = str1.split(";")[0]
      if model not in self.models:
        self.models.append( model )
        
    if num==0 or num>len(self.models):
      ch = Choice(self.models, "Please choose model :")
      model = ch[0]
    else:
      model = self.models[num-1]
    
    print "Loading data for :", model,
    sys.stdout.flush()
    
    self.allecus = []
    for str1 in self.vhcls:
      str1 = str1.strip()
      vrow = str1.split(";")
      for str2 in self.ecus:
        str2 = str2.strip()
        erow = str2.split(";")
        if vrow[0]==model and vrow[6]==erow[0]:
          str3 = ""
          for v in vrow[:6]:
            str3 = str3+v+self.s
          for v in erow:
            str3 = str3+v+self.s
          self.allecus.append(str3)
    print "  - "+str(len(self.allecus))+" ecus loaded"

  def compare_ecu( self, row, rrsp ):
    base = 19
    res = 0
    att = 0
    while base+2<len(row) and (int(row[base])*3+2)<len(rrsp):
      byte = int(rrsp[int(row[base])*3:int(row[base])*3+2],16)
      mask = int(row[base+1],16)
      val  = int(row[base+2],16)
      if (byte&mask)==val:
        res += 1
      att += 1
      base += 6
    if res==att and res>0:
      return True
    else:
      return False
      
  def request_can( self, row ):
    
    self.elm.cmd(row[11]) 			    #open session    
    rrsp = self.elm.cmd(row[16])		#get identification data
    if len(row[15])>1: 
      rerr = self.elm.cmd(row[15])	    #get errors
    else: rerr = ''
    
    self.elm.cmd(row[13])				#close session

    trerr = '''17ff00
0: 57 02 25 02 2E 00 
1: 10 D3 53 49 4E 45 45 

'''  
    trrsp = '''2180
01A 
0: 61 80 82 00 44 66 
1: 27 44 32 31 33 82 00 
2: 38 71 38 00 A7 74 00 
3: 56 05 02 01 00 00 00 

'''
  
    res = ""
    for s in rrsp.split('\n'):
      if ':' in s:
        res += s[2:].strip()+' '
    rrsp = res
    res = ""
    for s in rerr.split('\n'):
      if ':' in s:
        res += s[2:].strip()+' '
      else:
        if "57 " in s:
          res = s.strip()
    rerr = res
    return rrsp,rerr
  
  def request_iso( self, row ):
    if len(row[9])!=2: return 'addr error','addr error' 
    rsp = self.elm.cmd("81")	 		    #init bus
    rrsp = ""
    rerr = ""
    if "ERROR" not in rsp:
      self.elm.cmd(row[11]) 				#open session
      rrsp = self.elm.cmd(row[16])			#get identification data
      if len(row[15])>1: 
        rerr = self.elm.cmd(row[15])		#get errors
      else: rerr = ''
      self.elm.cmd(row[13])					#close session
    
    trerr = '''17FF00
57 06 90 7C 40 90 14 41 90 34 41 90 42 52 90 07 41 90 08 41 
>
'''
      
    trrsp = '''2180
61 80 82 00 44 66 27 44 32 31 33 82 00 38 71 38 00 A7 74 00 56 05 02 01 00 00 EF 
>
'''
    res = ""
    for s in rrsp.split('\n'):
      if ' F1 ' in s:
        res += s[2:].strip()+' '
    rrsp = res
    res = ""
    for s in rerr.split('\n'):
      if ' F1 ' in s:
        res += s[2:].strip()+' '
    rerr = res
    return rrsp,rerr
    
  def scan_can( self, row ):
    rrsp = ''
    rerr = ''
    for r in self.reqres:
      if (row[9]+row[11]+row[16])==r[0]:
        rrsp = r[1]
        rerr = r[2]
    if rrsp=='':
      rrsp,rerr = self.request_can( row )
      self.reqres.append([row[9]+row[11]+row[16],rrsp,rerr])
  
    compres = False
    if 'ERROR' not in rrsp:
      rrsp = rrsp[3:]
      compres = self.compare_ecu( row, rrsp )

    if compres:
      familynotdeteced = True
      for i in self.detectedEcus:
        if i[0]==row[5]:
          familynotdeteced = False
      if familynotdeteced:    
        self.detectedEcus.append([row[5],row[6],row[7],rerr,row[9],row[1]])
    
  def scan_iso( self, row ):
    rrsp = ''
    rerr = ''
    for r in self.reqres:
      if (row[9]+row[11]+row[16])==r[0]:
        rrsp = r[1]
        rerr = r[2]
    if rrsp=='':
      rrsp,rerr = self.request_iso( row )
      self.reqres.append([row[9]+row[11]+row[16],rrsp,rerr])
    
    compres = False
    if 'ERROR' not in rrsp:
      rrsp = rrsp[3:]
      compres = self.compare_ecu( row, rrsp )

    if not rerr: rerr = ''

    if compres:
      familynotdeteced = True
      for i in self.detectedEcus:
        if i[0]==row[5]:
          familynotdeteced = False
      if familynotdeteced:    
        self.detectedEcus.append([row[5],row[6],row[7],rerr,row[9],row[1]])



if __name__ == "__main__":
  ce = CarEcus()
  ce.chooseModel( 0 )
  for l in allecus:
    print l
