#!/usr/bin/env python
'''module contains class for working with ELM327'''

import serial                                    #sudo easy_install pyserial
import sys
import os
import io
import time
from datetime import datetime

snat = {"01":"760", "04":"762", "07":"771", "08":"778", "0D":"775", "0E":"76E", "0F":"770", 
		"13":"732", "1C":"76B", "1E":"768", "23":"773", "26":"765", "27":"76D", "29":"764", 
		"2A":"76F", "2C":"772", "3A":"7D2", "4D":"7BD", "50":"738", "51":"763", "57":"767", 
		"59":"734", "62":"7DD", "67":"793", "68":"77E", "6B":"7B5", "6E":"7E9", "77":"7DA", 
		"79":"7EA", "7A":"7E8", "7C":"77C", "86":"7A2", "87":"7A0", "93":"7BB", "95":"7EC", 
		"A5":"725", "A6":"726", "A8":"7B6", "C0":"7B9", "D1":"7EE", "F7":"736", "F8":"737", 
		"FA":"77B", "FD":"76F", "FE":"76C", "FF":"7D0"}
dnat = {"01":"740", "04":"742", "07":"751", "08":"758", "0D":"755", "0E":"74E", "0F":"750", 
		"13":"712", "1C":"74B", "1E":"748", "23":"753", "26":"745", "27":"74D", "29":"744", 
		"2A":"74F", "2C":"752", "3A":"7D6", "4D":"79D", "50":"718", "51":"743", "57":"747", 
		"59":"714", "62":"7DC", "67":"792", "68":"75A", "6B":"795", "6E":"7E1", "77":"7CA", 
		"79":"7E2", "7A":"7E0", "7C":"75C", "86":"782", "87":"780", "93":"79B", "95":"7E4", 
		"A5":"705", "A6":"706", "A8":"796", "C0":"799", "D1":"7E6", "F7":"716", "F8":"717", 
		"FA":"75B", "FD":"74F", "FE":"74C", "FF":"7D0"} 

class ELM:
  '''ELM327 class'''
  
  comp = 0
  lf   = 0
  buff = ""
    
  def __init__(self, port, speed, log):
    self.comp = serial.Serial(port,
                        baudrate=speed,
                        timeout=3)
    if len(log)>0:
      self.lf = open(log, "at")

  def cmd(self, command):
    
    if self.lf!=0:
      tm = str(time.time())
      tmstr = datetime.now().strftime("%H:%M:%S.%f")
      self.lf.write("["+tmstr+"]>"+command+"\n")
    self.comp.write(command+"\r")    

    self.buff = ""
    tb = time.time()
    tc = tb
    while(True):
      byte = self.comp.read()
      if byte=='\r': byte='\n'
      self.buff += byte
      tc = time.time()
      if byte=='>':
        break
      if (tc-tb)>self.comp.timeout:
        self.buff += "TIMEOUT"
        break
              
    if self.lf!=0:
      tm = str(time.time())
      self.lf.write("["+str(tc-tb)+"]<"+self.buff+"\n")
    return self.buff

  def close_protocol(self):
    self.cmd("atpc")
  
  def init_can(self):
    self.cmd("at z")
    self.cmd("at e1")
    self.cmd("at l1")
    self.cmd("at al")
    self.cmd("at st ff")
    self.cmd("at at 0")
    self.cmd("at sp 6")
    self.cmd("at at 1")

  def init_iso(self):
    self.cmd("at z")
    self.cmd("at l1")
    self.cmd("at d1")
    self.cmd("at st ff")
    self.cmd("at at 0")
    self.cmd("at sp 5")
    self.cmd("at at 1")
  
  def set_can_addr( self, addr ):
    TXa = dnat[addr]
    RXa = snat[addr]
    
    self.cmd("at sh "+TXa)
    self.cmd("at cra "+RXa)
    self.cmd("at fc sh "+TXa)
    self.cmd("at fc sd 30 00 00") # status BS STmin
    self.cmd("at fc sm 1")
    self.cmd("at sp 6")
  
  def set_iso_addr( self, addr ):
    self.cmd("at sh 82 "+addr+" f1")  #set address
    self.cmd("at sp 5")
  
