#!/usr/bin/python

import serial                                    #sudo easy_install pyserial
import sys, os

from serial.tools import list_ports
from mod_elm      import ELM
from mod_carecus  import CarEcus
#from mod_language import Language
#from mod_ecu      import ECU
#from mod_optfile  import *


opt_port  = ""
opt_speed = ""
opt_lang  = ""
opt_car   = ""
opt_ecu   = ""
opt_log   = ""

def optParser():
  '''Parsing of command line parameters. User should define at least com port name'''
  
  import optparse

  global  opt_port  
  global  opt_speed 
  global  opt_lang  
  global  opt_car   
  global  opt_ecu   
  global  opt_log   

  parser = optparse.OptionParser(
    usage = "%prog -p <port> [options]",
    version="%prog 0.1(alpha)",
    description = "pyRen - python version of Reanult car diagnostic"
  )
  
  parser.add_option("-p", "--port",
      help="ELM327 com port name",
      dest="port",
      type="string")

  parser.add_option("-s", "--speed",
      help="com port speed",
      dest="speed",
      default="38400",
      type="string")

  parser.add_option("-l", "--lang",
      help="language option",
      dest="lang",
      default="RU")

  parser.add_option("-m", "--model",
      help="number of car model",
      dest="car",
      default=0,
      type="int")

  parser.add_option("-e", "--ecu",
      help="number of ECU begining with family as a 0110016",
      dest="ecu",
      default="",
      type="string")

  parser.add_option("--log",
      help="log file name",
      dest="logfile",
      default="",
      type="string")

  (options, args) = parser.parse_args()
  
  if not options.port:
    parser.print_help()
    iterator = sorted(list(list_ports.comports()))
    print ""
    print "Available COM ports:"
    for port, desc, hwid in iterator:
      print "%-30s \n\tdesc: %s \n\thwid: %s" % (port,desc,hwid)
    print ""
    exit(2)
  else:
    opt_port  = options.port
    opt_speed = options.speed
    opt_lang  = options.lang
    opt_car   = options.car
    opt_ecu   = options.ecu
    opt_log   = options.logfile

def main():
  '''Main function'''

  global  opt_port  
  global  opt_speed 
  global  opt_lang  
  global  opt_car   
  global  opt_ecu   
  global  opt_log   
  global  tran
  
  optParser()
  
  elm = ELM( opt_port, int(opt_speed), opt_log )

  ce  = CarEcus(elm)                     #object containing lists of car ecus
  ce.chooseModel( opt_car )              #choose model of car for doing full scan

  #print "Loading language"
                                         #loading language data
  #lang = optfile("DiagOnCan_"+opt_lang+".bqm")
  
  ce.scanAllEcus()                       #First scan of all ecus
  
  #while( 1 ):
    #os.system('cls' if os.name == 'nt' else 'clear')    # clear screen
  current_ecu = ce.chooseECU( opt_ecu )               # choose ECU among detected
    #ecu = ECU(current_ecu, lang.dict)                   # loading ECU data
    #ecu.show_screens()                                  # show ECU screens

if __name__ == '__main__':
  main()
  